%Code associated with section V.A of https://hal.inria.fr/hal-01708695.
%
%This code runs with GraSP 1.1 (https://gforge.inria.fr/projects/grasp).
%
% Authors:
%  - Benjamin Girault <benjamin.girault@usc.edu>

% Copyright Benjamin Girault, University of Sourthern California, Los
% Angeles, California, USA (2018)
% 
% benjamin.girault@usc.edu
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use, 
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info". 
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability. 
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or 
% data to be ensured and,  more generally, to use and operate it in the 
% same conditions as regards security. 
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.

%% General
dir = 'MatlabData/';
csvWriteFlag = true;

Margin  = 0.004;
Spacing = 0;
Padding = [0.01 0.01 0.02 0.02];

%% 2-Class Graph
% Uncomment these lines to generate a new graph

% g = grasp_struct;
% 
% N_large = 300;
% N_sparse = 30;
% K = 2;
% 
% g.layout((1:N_large)', 1) = normrnd(8, .7, N_large, 1);
% g.layout((1:N_large)', 2) = normrnd(5, 1.5, N_large, 1);
% 
% g.layout(N_large + (1:N_sparse)', 1) = normrnd(4, 2, N_sparse, 1);
% g.layout(N_large + (1:N_sparse)', 2) = normrnd(5, 1.7, N_sparse, 1);
% 
% g.distances = grasp_distances_layout(g);
% 
% indicator = [ones(1, N_large) 2 * ones(1, N_sparse)]';
% 
% save('g_clustering_2class.mat', 'g', 'indicator', 'K');

%% Load graph

g = load('g_clustering_2class.mat');
indicator = g.indicator;

K = g.K;
g = g.g;

g.A = grasp_adjacency_gaussian(g, .4);
g.A = grasp_adjacency_knn(g, 10);

if csvWriteFlag
    grasp_exportcsv(g, [dir 'clustering_nodes.csv'], [dir 'clustering_edges.csv']);
end

grasp_open_figure_name('Ground Truth');
grasp_show_graph(gca, g, 'show_edges', true, 'node_values', indicator);

if csvWriteFlag
    grasp_exportcsv_signal(g, indicator, [dir 'clustering_classes.csv']);
end

grasp_open_figure_name('Degree');
grasp_show_graph(gca, g, 'show_edges', true, 'node_values', diag(grasp_degrees(g)));

if csvWriteFlag
    grasp_exportcsv_signal(g, diag(grasp_degrees(g)), [dir 'clustering_degrees.csv']);
end

%% 

fprintf('Approach\t\tAccuracy\tF1 sparse\tF1 dense\n');

%% Clustering from the combinatorial Laplacian

nbModesKMeans = K;
nbKMeans = K;
nbApproaches = 7;


g_L = grasp_eigendecomposition(g);
curApproach = 1;

nbModesSkip = 1;
idx_LI = kmeans(g_L.Finv(:, (nbModesSkip + 1):nbModesKMeans), nbKMeans);
if sum(idx_LI == 2) > sum(idx_LI == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idx_LI = (idx_LI == 2) + 2 * (idx_LI == 1);
end
positive_sparse = (indicator == 2) & (idx_LI == 2);
negative_sparse = (indicator == 2) & (idx_LI == 1);
positive_dense  = (indicator == 1) & (idx_LI == 1);
negative_dense  = (indicator == 1) & (idx_LI == 2);


fprintf('(L, I)\t\t\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idx_LI == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_open_figure_name('Clustering');
grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idx_LI);
title('L');

if csvWriteFlag
    grasp_exportcsv_signal(g, idx_LI, [dir 'clustering_L.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(ones(sum(positive_sparse), 1) / sqrt(grasp_nb_nodes(g)), g_L.Finv(positive_sparse, nbModesSkip + 1), 'ro',...
     ones(sum(negative_sparse), 1) / sqrt(grasp_nb_nodes(g)), g_L.Finv(negative_sparse, nbModesSkip + 1), 'bx',...
     ones(sum(positive_dense),  1) / sqrt(grasp_nb_nodes(g)), g_L.Finv(positive_dense,  nbModesSkip + 1), 'bo',...
     ones(sum(negative_dense),  1) / sqrt(grasp_nb_nodes(g)), g_L.Finv(negative_dense,  nbModesSkip + 1), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_L_feature_pos_spar.csv'], [ones(sum(positive_sparse), 1) / sqrt(grasp_nb_nodes(g)) g_L.Finv(positive_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_L_feature_neg_spar.csv'], [ones(sum(negative_sparse), 1) / sqrt(grasp_nb_nodes(g)) g_L.Finv(negative_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_L_feature_pos_dens.csv'], [ones(sum(positive_dense),  1) / sqrt(grasp_nb_nodes(g)) g_L.Finv(positive_dense,  nbModesSkip + 1)]);
    csvwrite([dir 'clustering_L_feature_neg_dens.csv'], [ones(sum(negative_dense),  1) / sqrt(grasp_nb_nodes(g)) g_L.Finv(negative_dense,  nbModesSkip + 1)]);
end

%% Clustering from the normalized Laplacian

g_NL = grasp_eigendecomposition(g, 'matrix', 'norm_lapl');
curApproach = 2;

T = g_NL.Finv(:, 1:nbModesKMeans);
if mean(T(:, 1)) < 0
    T(:, 1) = -T(:, 1);
end

idx_NL = kmeans(T, nbKMeans);
if sum(idx_NL == 2) > sum(idx_NL == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idx_NL = (idx_NL == 2) + 2 * (idx_NL == 1);
end
positive_sparse = (indicator == 2) & (idx_NL == 2);
negative_sparse = (indicator == 2) & (idx_NL == 1);
positive_dense  = (indicator == 1) & (idx_NL == 1);
negative_dense  = (indicator == 1) & (idx_NL == 2);

fprintf('(Lnorm, I)\t\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idx_NL == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idx_NL);
title('$\mathcal{L}$', 'Interpreter', 'Latex');

if csvWriteFlag
    grasp_exportcsv_signal(g, idx_NL, [dir 'clustering_Lnorm.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(T(positive_sparse, 1), T(positive_sparse, 2), 'ro',...
     T(negative_sparse, 1), T(negative_sparse, 2), 'bx',...
     T(positive_dense,  1), T(positive_dense,  2), 'bo',...
     T(negative_dense,  1), T(negative_dense,  2), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_Lnorm_feature_pos_spar.csv'], [T(positive_sparse, 1) T(positive_sparse, 2)]);
    csvwrite([dir 'clustering_Lnorm_feature_neg_spar.csv'], [T(negative_sparse, 1) T(negative_sparse, 2)]);
    csvwrite([dir 'clustering_Lnorm_feature_pos_dens.csv'], [T(positive_dense,  1) T(positive_dense,  2)]);
    csvwrite([dir 'clustering_Lnorm_feature_neg_dens.csv'], [T(negative_dense,  1) T(negative_dense,  2)]);
end

%% Clustering from the normalized Laplacian (original)

g_NL = grasp_eigendecomposition(g, 'matrix', 'norm_lapl');
curApproach = 3;

T = diag(arrayfun(@(i) norm(g_NL.Finv(i, 1:nbModesKMeans), 2), 1:grasp_nb_nodes(g)) .^ -1) * g_NL.Finv(:, 1:nbModesKMeans);
if mean(T(:, 1)) < 0
    T(:, 1) = -T(:, 1);
end

idx_NLorig = kmeans(T, nbKMeans);
if sum(idx_NLorig == 2) > sum(idx_NLorig == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idx_NLorig = (idx_NLorig == 2) + 2 * (idx_NLorig == 1);
end
positive_sparse = (indicator == 2) & (idx_NLorig == 2);
negative_sparse = (indicator == 2) & (idx_NLorig == 1);
positive_dense  = (indicator == 1) & (idx_NLorig == 1);
negative_dense  = (indicator == 1) & (idx_NLorig == 2);

fprintf('(Lnorm, I), feat. norm.\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idx_NLorig == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idx_NLorig);
title('$\mathcal{L}$ (original)', 'Interpreter', 'Latex');

if csvWriteFlag
    grasp_exportcsv_signal(g, idx_NLorig, [dir 'clustering_Lnorm_orig.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(T(positive_sparse, 1), T(positive_sparse, 2), 'ro',...
     T(negative_sparse, 1), T(negative_sparse, 2), 'bx',...
     T(positive_dense,  1), T(positive_dense,  2), 'bo',...
     T(negative_dense,  1), T(negative_dense,  2), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_Lnorm_orig_feature_pos_spar.csv'], [T(positive_sparse, 1) T(positive_sparse, 2)]);
    csvwrite([dir 'clustering_Lnorm_orig_feature_neg_spar.csv'], [T(negative_sparse, 1) T(negative_sparse, 2)]);
    csvwrite([dir 'clustering_Lnorm_orig_feature_pos_dens.csv'], [T(positive_dense,  1) T(positive_dense,  2)]);
    csvwrite([dir 'clustering_Lnorm_orig_feature_neg_dens.csv'], [T(negative_dense,  1) T(negative_dense,  2)]);
end

%% Clustering from the random walk Laplacian

g_LRW = grasp_eigendecomposition(g, 'inner_product', grasp_degrees(g));
if mean(g_LRW.Finv(:, 2)) < 0
    g_LRW.Finv(:, 2) = -g_LRW.Finv(:, 2);
end

curApproach = 4;

nbModesSkip = 1;
idxLRW = kmeans(g_LRW.Finv(:, (nbModesSkip + 1):nbModesKMeans), nbKMeans);
if sum(idxLRW == 2) > sum(idxLRW == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idxLRW = (idxLRW == 2) + 2 * (idxLRW == 1);
end
positive_sparse = (indicator == 2) & (idxLRW == 2);
negative_sparse = (indicator == 2) & (idxLRW == 1);
positive_dense  = (indicator == 1) & (idxLRW == 1);
negative_dense  = (indicator == 1) & (idxLRW == 2);

fprintf('(L,D)\t\t\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idxLRW == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idxLRW);
title('LRW');

if csvWriteFlag
    grasp_exportcsv_signal(g, idxLRW, [dir 'clustering_LRW.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(ones(sum(positive_sparse), 1) / sqrt(sum(g_LRW.Q(:))), g_LRW.Finv(positive_sparse, nbModesSkip + 1), 'ro',...
     ones(sum(negative_sparse), 1) / sqrt(sum(g_LRW.Q(:))), g_LRW.Finv(negative_sparse, nbModesSkip + 1), 'bx',...
     ones(sum(positive_dense),  1) / sqrt(sum(g_LRW.Q(:))), g_LRW.Finv(positive_dense,  nbModesSkip + 1), 'bo',...
     ones(sum(negative_dense),  1) / sqrt(sum(g_LRW.Q(:))), g_LRW.Finv(negative_dense,  nbModesSkip + 1), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_LRW_feature_pos_spar.csv'], [ones(sum(positive_sparse), 1) / sqrt(sum(g_LRW.Q(:))) g_LRW.Finv(positive_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_LRW_feature_neg_spar.csv'], [ones(sum(negative_sparse), 1) / sqrt(sum(g_LRW.Q(:))) g_LRW.Finv(negative_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_LRW_feature_pos_dens.csv'], [ones(sum(positive_dense),  1) / sqrt(sum(g_LRW.Q(:))) g_LRW.Finv(positive_dense,  nbModesSkip + 1)]);
    csvwrite([dir 'clustering_LRW_feature_neg_dens.csv'], [ones(sum(negative_dense),  1) / sqrt(sum(g_LRW.Q(:))) g_LRW.Finv(negative_dense,  nbModesSkip + 1)]);
end

%% Clustering from an extreme Laplacian (Q=D^-2)

g_Ls = grasp_eigendecomposition(g, 'inner_product', diag(diag(grasp_degrees(g)) .^ -2));
curApproach = 5;

nbModesSkip = 1;
idxLS = kmeans(g_Ls.Finv(:, (nbModesSkip + 1):nbModesKMeans), nbKMeans);
if sum(idxLS == 2) > sum(idxLS == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idxLS = (idxLS == 2) + 2 * (idxLS == 1);
end
positive_sparse = (indicator == 2) & (idxLS == 2);
negative_sparse = (indicator == 2) & (idxLS == 1);
positive_dense  = (indicator == 1) & (idxLS == 1);
negative_dense  = (indicator == 1) & (idxLS == 2);

fprintf('(L,D^-2)\t\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idxLS == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idxLS);
title('Ls');

if csvWriteFlag
    grasp_exportcsv_signal(g, idxLS, [dir 'clustering_LD-2.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(ones(sum(positive_sparse), 1) / sqrt(sum(g_Ls.Q(:))), g_Ls.Finv(positive_sparse, nbModesSkip + 1), 'ro',...
     ones(sum(negative_sparse), 1) / sqrt(sum(g_Ls.Q(:))), g_Ls.Finv(negative_sparse, nbModesSkip + 1), 'bx',...
     ones(sum(positive_dense),  1) / sqrt(sum(g_Ls.Q(:))), g_Ls.Finv(positive_dense,  nbModesSkip + 1), 'bo',...
     ones(sum(negative_dense),  1) / sqrt(sum(g_Ls.Q(:))), g_Ls.Finv(negative_dense,  nbModesSkip + 1), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_LD-2_feature_pos_spar.csv'], [ones(sum(positive_sparse), 1) / sqrt(sum(g_Ls.Q(:))) g_Ls.Finv(positive_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_LD-2_feature_neg_spar.csv'], [ones(sum(negative_sparse), 1) / sqrt(sum(g_Ls.Q(:))) g_Ls.Finv(negative_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_LD-2_feature_pos_dens.csv'], [ones(sum(positive_dense),  1) / sqrt(sum(g_Ls.Q(:))) g_Ls.Finv(positive_dense,  nbModesSkip + 1)]);
    csvwrite([dir 'clustering_LD-2_feature_neg_dens.csv'], [ones(sum(negative_dense),  1) / sqrt(sum(g_Ls.Q(:))) g_Ls.Finv(negative_dense,  nbModesSkip + 1)]);
end

%% Clustering from the Voronoi norm (Q=C)

g_Lc = grasp_eigendecomposition(g, 'inner_product', diag(grasp_voronoi_areas(g)));
curApproach = 6;

nbModesSkip = 1;
idxLC = kmeans(g_Lc.Finv(:, (nbModesSkip + 1):nbModesKMeans), nbKMeans);
if sum(idxLC == 2) > sum(idxLC == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idxLC = (idxLC == 2) + 2 * (idxLC == 1);
end
positive_sparse = (indicator == 2) & (idxLC == 2);
negative_sparse = (indicator == 2) & (idxLC == 1);
positive_dense  = (indicator == 1) & (idxLC == 1);
negative_dense  = (indicator == 1) & (idxLC == 2);

fprintf('(L,C)\t\t\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idxLC == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idxLC);
title('Lc');

if csvWriteFlag
    grasp_exportcsv_signal(g, idxLC, [dir 'clustering_Lc.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(ones(sum(positive_sparse), 1) / sqrt(sum(g_Lc.Q(:))), g_Ls.Finv(positive_sparse, nbModesSkip + 1), 'ro',...
     ones(sum(negative_sparse), 1) / sqrt(sum(g_Lc.Q(:))), g_Ls.Finv(negative_sparse, nbModesSkip + 1), 'bx',...
     ones(sum(positive_dense),  1) / sqrt(sum(g_Lc.Q(:))), g_Ls.Finv(positive_dense,  nbModesSkip + 1), 'bo',...
     ones(sum(negative_dense),  1) / sqrt(sum(g_Lc.Q(:))), g_Ls.Finv(negative_dense,  nbModesSkip + 1), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_Lc_feature_pos_spar.csv'], [ones(sum(positive_sparse), 1) / sqrt(sum(g_Lc.Q(:))) g_Ls.Finv(positive_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_Lc_feature_neg_spar.csv'], [ones(sum(negative_sparse), 1) / sqrt(sum(g_Lc.Q(:))) g_Ls.Finv(negative_sparse, nbModesSkip + 1)]);
    csvwrite([dir 'clustering_Lc_feature_pos_dens.csv'], [ones(sum(positive_dense),  1) / sqrt(sum(g_Lc.Q(:))) g_Ls.Finv(positive_dense,  nbModesSkip + 1)]);
    csvwrite([dir 'clustering_Lc_feature_neg_dens.csv'], [ones(sum(negative_dense),  1) / sqrt(sum(g_Lc.Q(:))) g_Ls.Finv(negative_dense,  nbModesSkip + 1)]);
end

%% Clustering using Sandryhaila & Moura approach: Delta(x)=|| x - Ax ||_1 (Q=I)

g_A1 = g;
g_A1.Anorm = g.A / eigs(g.A, 1, 'lm');
g_A1.Finv = grasp_gfm_l1(g_A1, eye(grasp_nb_nodes(g)) - g_A1.Anorm, eye(grasp_nb_nodes(g)), nbModesKMeans); % Q=I
% g_A1.Finv = grasp_gfm_l1(g_A1, eye(grasp_nb_nodes(g)) - g_A1.Anorm, grasp_degrees(g), nbModesKMeans); % Q=D
if mean(g_A1.Finv(:, 1)) < 0
    g_A1.Finv(:, 1) = -g_A1.Finv(:, 1);
end
if mean(g_A1.Finv(:, 2)) < 0
    g_A1.Finv(:, 2) = -g_A1.Finv(:, 2);
end

curApproach = 7;

idxA1 = kmeans(g_A1.Finv(:, 1:nbModesKMeans), nbKMeans);
if sum(idxA1 == 2) > sum(idxA1 == 1)
    % Swap the cluster indices for cluster 1 to be the largest one
    idxA1 = (idxA1 == 2) + 2 * (idxA1 == 1);
end
positive_sparse = (indicator == 2) & (idxA1 == 2);
negative_sparse = (indicator == 2) & (idxA1 == 1);
positive_dense  = (indicator == 1) & (idxA1 == 1);
negative_dense  = (indicator == 1) & (idxA1 == 2);

fprintf('(||x-Ax||_1, I)\t\t%05.2f%%\t\t%05.2f%%\t\t%05.2f%%\n',...
            sum(idxA1 == indicator) / grasp_nb_nodes(g) * 100,...
            2 * sum(positive_sparse) / (2 * sum(positive_sparse) + sum(negative_sparse) + sum(negative_dense)) * 100,...
            2 * sum(positive_dense) / (2 * sum(positive_dense) + sum(negative_sparse) + sum(negative_dense)) * 100);

grasp_subaxis(2, nbApproaches, curApproach, 'M', [Margin Margin Margin * 2 Margin], 'S', Spacing, 'P', Padding);
grasp_show_graph(gca, g, 'show_edges', false, 'node_values', idxA1);
title('A (l1)');

if csvWriteFlag
    grasp_exportcsv_signal(g, idxA1, [dir 'clustering_A_l1.csv']);
end

grasp_subaxis(2, nbApproaches, nbApproaches + curApproach, 'M', [Margin Margin Margin Margin * 10], 'S', Spacing, 'P', Padding);
plot(g_A1.Finv(positive_sparse, 1), g_A1.Finv(positive_sparse, 2), 'ro',...
     g_A1.Finv(negative_sparse, 1), g_A1.Finv(negative_sparse, 2), 'bx',...
     g_A1.Finv(positive_dense,  1), g_A1.Finv(positive_dense,  2), 'bo',...
     g_A1.Finv(negative_dense,  1), g_A1.Finv(negative_dense,  2), 'rx');
xlabel('u0');
ylabel('u1');

if csvWriteFlag
    csvwrite([dir 'clustering_A_l1_feature_pos_spar.csv'], [g_A1.Finv(positive_sparse, 1) g_A1.Finv(positive_sparse, 2)]);
    csvwrite([dir 'clustering_A_l1_feature_neg_spar.csv'], [g_A1.Finv(negative_sparse, 1) g_A1.Finv(negative_sparse, 2)]);
    csvwrite([dir 'clustering_A_l1_feature_pos_dens.csv'], [g_A1.Finv(positive_dense,  1) g_A1.Finv(positive_dense,  2)]);
    csvwrite([dir 'clustering_A_l1_feature_neg_dens.csv'], [g_A1.Finv(negative_dense,  1) g_A1.Finv(negative_dense,  2)]);
end

%% NCuts

DeltaQuad = @(g, x) x' * g.L * x / (x' * g.Q * x) / 2;
DeltaA1   = @(x) norm((eye(grasp_nb_nodes(g)) - g_A1.Anorm) * x, 1) / norm(x);

NcutL   = @(x) DeltaQuad(g_L,   double(x == 1)) + DeltaQuad(g_L,   double(x == 2));
NcutNL  = @(x) DeltaQuad(g_NL,  double(x == 1)) + DeltaQuad(g_NL,  double(x == 2));
NcutLRW = @(x) DeltaQuad(g_LRW, double(x == 1)) + DeltaQuad(g_LRW, double(x == 2));
NcutLS  = @(x) DeltaQuad(g_Ls,  double(x == 1)) + DeltaQuad(g_Ls,  double(x == 2));
NcutLC  = @(x) DeltaQuad(g_Lc,  double(x == 1)) + DeltaQuad(g_Lc,  double(x == 2));
NcutA1  = @(x) DeltaA1(double(x == 1))          + DeltaA1(double(x == 2));

fprintf('\nSolution \\ NCut\t(L,I)\t(Lnorm,I)\t(L,D)\t(L,D^-2)\t(L,C)\t(A_l1,I)\n');

fprintf('(L,I)\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    NcutL(idx_LI), NcutNL(idx_LI), NcutLRW(idx_LI), NcutLS(idx_LI), NcutLC(idx_LI), NcutA1(idx_LI));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idx_LI == 1)),...
    DeltaQuad(g_NL,  double(idx_LI == 1)),...
    DeltaQuad(g_LRW, double(idx_LI == 1)),...
    DeltaQuad(g_Ls,  double(idx_LI == 1)),...
    DeltaQuad(g_Lc,  double(idx_LI == 1)),...
    DeltaA1(double(idx_LI == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idx_LI == 2)),...
    DeltaQuad(g_NL,  double(idx_LI == 2)),...
    DeltaQuad(g_LRW, double(idx_LI == 2)),...
    DeltaQuad(g_Ls,  double(idx_LI == 2)),...
    DeltaQuad(g_Lc,  double(idx_LI == 2)),...
    DeltaA1(double(idx_LI == 2)));
fprintf('\n');

fprintf('(Lnorm,I)\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    NcutL(idx_NL), NcutNL(idx_NL), NcutLRW(idx_NL), NcutLS(idx_NL), NcutLC(idx_NL), NcutA1(idx_NL));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idx_NL == 1)),...
    DeltaQuad(g_NL,  double(idx_NL == 1)),...
    DeltaQuad(g_LRW, double(idx_NL == 1)),...
    DeltaQuad(g_Ls,  double(idx_NL == 1)),...
    DeltaQuad(g_Lc,  double(idx_NL == 1)),...
    DeltaA1(double(idx_NL == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idx_NL == 2)),...
    DeltaQuad(g_NL,  double(idx_NL == 2)),...
    DeltaQuad(g_LRW, double(idx_NL == 2)),...
    DeltaQuad(g_Ls,  double(idx_NL == 2)),...
    DeltaQuad(g_Lc,  double(idx_NL == 2)),...
    DeltaA1(double(idx_NL == 2)));
fprintf('\n');

fprintf('(Lnorm,I)_orig\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    NcutL(idx_NLorig), NcutNL(idx_NLorig), NcutLRW(idx_NLorig), NcutLS(idx_NLorig), NcutLC(idx_NLorig), NcutA1(idx_NLorig));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idx_NLorig == 1)),...
    DeltaQuad(g_NL,  double(idx_NLorig == 1)),...
    DeltaQuad(g_LRW, double(idx_NLorig == 1)),...
    DeltaQuad(g_Ls,  double(idx_NLorig == 1)),...
    DeltaQuad(g_Lc,  double(idx_NLorig == 1)),...
    DeltaA1(double(idx_NLorig == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idx_NLorig == 2)),...
    DeltaQuad(g_NL,  double(idx_NLorig == 2)),...
    DeltaQuad(g_LRW, double(idx_NLorig == 2)),...
    DeltaQuad(g_Ls,  double(idx_NLorig == 2)),...
    DeltaQuad(g_Lc,  double(idx_NLorig == 2)),...
    DeltaA1(double(idx_NLorig == 2)));
fprintf('\n');

fprintf('(L,D)\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    NcutL(idxLRW), NcutNL(idxLRW), NcutLRW(idxLRW), NcutLS(idxLRW), NcutLC(idxLRW), NcutA1(idxLRW));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxLRW == 1)),...
    DeltaQuad(g_NL,  double(idxLRW == 1)),...
    DeltaQuad(g_LRW, double(idxLRW == 1)),...
    DeltaQuad(g_Ls,  double(idxLRW == 1)),...
    DeltaQuad(g_Lc,  double(idxLRW == 1)),...
    DeltaA1(double(idxLRW == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxLRW == 2)),...
    DeltaQuad(g_NL,  double(idxLRW == 2)),...
    DeltaQuad(g_LRW, double(idxLRW == 2)),...
    DeltaQuad(g_Ls,  double(idxLRW == 2)),...
    DeltaQuad(g_Lc,  double(idxLRW == 2)),...
    DeltaA1(double(idxLRW == 2)));
fprintf('\n');

fprintf('(L,D^-2)\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    NcutL(idxLS), NcutNL(idxLS), NcutLRW(idxLS), NcutLS(idxLS), NcutLC(idxLS), NcutA1(idxLS));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxLS == 1)),...
    DeltaQuad(g_NL,  double(idxLS == 1)),...
    DeltaQuad(g_LRW, double(idxLS == 1)),...
    DeltaQuad(g_Ls,  double(idxLS == 1)),...
    DeltaQuad(g_Lc,  double(idxLS == 1)),...
    DeltaA1(double(idxLS == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxLS == 2)),...
    DeltaQuad(g_NL,  double(idxLS == 2)),...
    DeltaQuad(g_LRW, double(idxLS == 2)),...
    DeltaQuad(g_Ls,  double(idxLS == 2)),...
    DeltaQuad(g_Lc,  double(idxLS == 2)),...
    DeltaA1(double(idxLS == 2)));
fprintf('\n');

fprintf('(L,C)\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    NcutL(idxLC), NcutNL(idxLC), NcutLRW(idxLC), NcutLS(idxLC), NcutLC(idxLC), NcutA1(idxLC));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxLC == 1)),...
    DeltaQuad(g_NL,  double(idxLC == 1)),...
    DeltaQuad(g_LRW, double(idxLC == 1)),...
    DeltaQuad(g_Ls,  double(idxLC == 1)),...
    DeltaQuad(g_Lc,  double(idxLC == 1)),...
    DeltaA1(double(idxLC == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxLC == 2)),...
    DeltaQuad(g_NL,  double(idxLC == 2)),...
    DeltaQuad(g_LRW, double(idxLC == 2)),...
    DeltaQuad(g_Ls,  double(idxLC == 2)),...
    DeltaQuad(g_Lc,  double(idxLC == 2)),...
    DeltaA1(double(idxLC == 2)));
fprintf('\n');

fprintf('(A_l1,I)\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t%05.4f\t%05.4f\n', ...
    NcutL(idxA1), NcutNL(idxA1), NcutLRW(idxA1), NcutLS(idxA1), NcutLC(idxA1), NcutA1(idxA1));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxA1 == 1)),...
    DeltaQuad(g_NL,  double(idxA1 == 1)),...
    DeltaQuad(g_LRW, double(idxA1 == 1)),...
    DeltaQuad(g_Ls,  double(idxA1 == 1)),...
    DeltaQuad(g_Lc,  double(idxA1 == 1)),...
    DeltaA1(double(idxA1 == 1)));
fprintf('\t\t%05.4f\t%05.4f\t\t%05.4f\t%05.4f\t%05.4f\t%05.4f\n', ...
    DeltaQuad(g_L,   double(idxA1 == 2)),...
    DeltaQuad(g_NL,  double(idxA1 == 2)),...
    DeltaQuad(g_LRW, double(idxA1 == 2)),...
    DeltaQuad(g_Ls,  double(idxA1 == 2)),...
    DeltaQuad(g_Lc,  double(idxA1 == 2)),...
    DeltaA1(double(idxA1 == 2)));


